/*
** HDL4SE: VerilogۺϷƽ̨
** Copyright (C) 2021-2021, raoxianhong<raoxianhong@163.net>
** LCOM: ģ
** Copyright (C) 2021-2021, raoxianhong<raoxianhong@163.net>
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are met:
**
** * Redistributions of source code must retain the above copyright notice,
**   this list of conditions and the following disclaimer.
** * Redistributions in binary form must reproduce the above copyright notice,
**   this list of conditions and the following disclaimer in the documentation
**   and/or other materials provided with the distribution.
** * The name of the author may be used to endorse or promote products
**   derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
** AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
** ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
** LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
** THE POSSIBILITY OF SUCH DAMAGE.
*/

/*
* bignumber.c
    202105201110: rxh, initial version
*/

#include "stdlib.h" 
#include "stdio.h"
#include "object.h"
#define IMPLEMENT_GUID
#include "bignumber.h"
#undef IMPLEMENT_GUID

#define CELL_WIDTH 32
#define CELL_MASK 0xffffffff

typedef struct _sBigInteger {
	OBJECT_HEADER
	INTERFACE_DECLARE(IBigNumber)
	BIGNUMBER_VARDECLARE

	int buflen;
	int width;
	unsigned int* buf;

}sBigInteger;

OBJECT_FUNCDECLARE(bigint, CLSID_BIGINTEGER);
BIGNUMBER_FUNCDECLARE(bigint, CLSID_BIGINTEGER, sBigInteger);

OBJECT_FUNCIMPL(bigint, sBigInteger, CLSID_BIGINTEGER);

QUERYINTERFACE_BEGIN(bigint, CLSID_BIGINTEGER)
QUERYINTERFACE_ITEM(IID_BIGNUMBER, IBigNumber, sBigInteger)
QUERYINTERFACE_END

static const char* bigintModuleInfo()
{
	return "1.0.0-20210520.1355 Big integer library";
}

static int bigintCreate(const PARAMITEM* pParams, int paramcount, HOBJECT* pObject)
{
	sBigInteger* pobj;
	int i;
	pobj = (sBigInteger*)malloc(sizeof(sBigInteger));
	if (pobj == NULL)
		return -1;
	*pObject = 0;
	BIGNUMBER_VARINIT(pobj, CLSID_BIGINTEGER);
	INTERFACE_INIT(IBigNumber, pobj, bigint, bn);

	pobj->width = 64;
	for (i = 0; i < paramcount; i++) {
		if (pParams[i].name == PARAMID_BIGINTEGERWIDTH)
			pobj->width = pParams[i].i32value;
	}

	pobj->buflen = (pobj->width + CELL_WIDTH - 1) / CELL_WIDTH;
	if (pobj->buflen < 2)
		pobj->buflen = 2;
	pobj->buf = malloc(pobj->buflen * sizeof(unsigned int));
	if (pobj->buf == NULL)
		return -1;

	for (i = 0;i<pobj->buflen;i++)
		pobj->buf[i] = 0;

	/* ɵĶ */
	OBJECT_RETURN_GEN(bigint, pobj, pObject, CLSID_BIGINTEGER);
	return EIID_OK;
}

static int actualwidth(unsigned int n)
{
	unsigned int c = 1;
	int ret = 1;
	if (n & 0x80000000)
		return CELL_WIDTH;
	while (c <= n) {
		c <<= 1;
		ret++;
	}
	return ret-1;
}

static void bigintDestroy(HOBJECT object)
{
	sBigInteger* pobj;
	pobj = (sBigInteger*)objectThis(object);
	free(pobj->buf);
	free(pobj);
}

static int bigintValid(HOBJECT object)
{
	sBigInteger* pobj;
	pobj = (sBigInteger*)objectThis(object);
	if (pobj == NULL)
		return 0;
	if (pobj->buflen < 2)
		return 0;
	if (pobj->buflen * CELL_WIDTH < pobj->width)
		return 0;
	if (pobj->buf == NULL)
		return 0;
	return 1;
}

static int bigint_bn_GetWidth(HOBJECT object)
{
	int i;
	sBigInteger* pobj;
	pobj = (sBigInteger*)objectThis(object);
	return pobj->width;
}

static int bigint_bn_SetWidth(HOBJECT object, int width, int signexpand)
{
	int i, bc,blen, sign;
	sBigInteger* pobj;
	pobj = (sBigInteger*)objectThis(object);
	if (width <= 0) {
		memset(pobj->buf, 0, pobj->buflen * (CELL_WIDTH/8));
		pobj->width = 0;
		return 0;
	}
	sign = 0;
	if (signexpand) {
		bc = pobj->width / CELL_WIDTH;
		sign = pobj->buf[bc] & (1 << ((pobj->width-1) & (CELL_WIDTH-1)));
	}

	blen = (width + CELL_WIDTH - 1) / CELL_WIDTH;
	if (pobj->buflen < blen) {
		unsigned int * buf;
		buf = (unsigned int *)malloc(blen * (CELL_WIDTH / 8));
		if (buf == NULL)
			return -1;
		for (i = 0; i < pobj->buflen; i++)
			buf[i] = pobj->buf[i];
		pobj->buflen = blen;
		free(pobj->buf);
		pobj->buf = buf;
	}
	
	blen = pobj->width / CELL_WIDTH;
	for (i = blen+1; i < pobj->buflen; i++) {
		pobj->buf[i] = sign ? CELL_MASK : 0;
	}
	if (sign)
		pobj->buf[blen] |= CELL_MASK << (pobj->width & (CELL_WIDTH - 1));
	else
		pobj->buf[blen] &= CELL_MASK >> (CELL_WIDTH - (pobj->width & (CELL_WIDTH-1)));
	pobj->width = width;
	return 0;
}

static int bigint_bn_GetInt(HOBJECT object, int* pvalue)
{ 
	sBigInteger* pobj;
	int width;
	pobj = (sBigInteger*)objectThis(object);
	width = pobj->width;
	if (width < 32)
		bigint_bn_SetWidth(object, 32, 1);
	*(unsigned int*)pvalue = pobj->buf[0];
	if (width < 32)
		bigint_bn_SetWidth(object, width, 0);
	return 0; 
}

static int bigint_bn_GetInt64(HOBJECT object, long long* pvalue)
{
	sBigInteger* pobj;
	int width;
	pobj = (sBigInteger*)objectThis(object);
	width = pobj->width;
	if (width < 64)
		bigint_bn_SetWidth(object, 64, 1);

	((unsigned int *)pvalue)[0] = pobj->buf[0];
	((unsigned int *)pvalue)[1] = pobj->buf[1];
	if (width < 64)
		bigint_bn_SetWidth(object, width, 0);

	return 0;
}

static int bigint_bn_GetStr(HOBJECT object, int base, char* str, int buflen)
{ 
	int i;
	int bc, ac;
	char fmt[10];
	char buf[10];
	sBigInteger* pobj;
	pobj = (sBigInteger*)objectThis(object);
	str[0] = 0;
	bc = bigint_bn_GetWidth(object);
	if (bc == 0) {
		strcpy(str, "0");
		return 0;
	}
	sprintf(str, "%d'h", bc);
	i = ((bc + (CELL_WIDTH-1)) / CELL_WIDTH) - 1;
	ac = actualwidth(pobj->buf[i]);
	ac = (ac + 7) / 8;
	sprintf(fmt, "%%%dx_", ac);
	sprintf(buf, fmt, pobj->buf[i]);
	strcat(str, buf);
	for (i = ((bc + (CELL_WIDTH-1)) / CELL_WIDTH)-2; i >= 0; i--) {
		sprintf(buf, "%08x", pobj->buf[i]);
		strcat(str, buf);
		if (i > 0)
			strcat(str, "_");
	}
	return 0; 
}

enum TOKEN_STATE {
	TOKEN_INITIAL,
	TOKEN_NUM,
	TOKEN_BASE,
	TOKEN_BIN,
	TOKEN_OCT,
	TOKEN_DEC,
	TOKEN_HEX
};

static int bigint_bn_AssignStr(HOBJECT object, const char* str, const char **nstr)
{ 
	enum TOKEN_STATE state;
	const char* strt = str;
	int numvalid = 0;
	sBigInteger* pobj;
	pobj = (sBigInteger*)objectThis(object);
	state = TOKEN_INITIAL;
	while (*strt != 0) {
		int ch = *strt;
		switch (state) {
		case TOKEN_INITIAL: {
			if (ch == '\'') {
				state = TOKEN_BASE;
			}
			else if (ch >= '0' && ch <= '9') {
				bigint_bn_AssignInt(object, ch - '0');
				numvalid = 1;
				state = TOKEN_NUM;
			}
		} break;
		case TOKEN_NUM: {
			if (ch >= '0' && ch <= '9') {
				bigint_bn_MulInt(object, 10);
				bigint_bn_AddInt(object, ch - '0');
			}
			else if (ch == '\'') {
				state = TOKEN_BASE;
				numvalid = 0;
			}
			else {
				goto lastnum;
			}
		}break;
		case TOKEN_BASE: {
			if (ch == 'b' || ch == 'B') {
				bigint_bn_AssignInt(object, 0);
				state = TOKEN_BIN;
			}
			else if (ch == 'h' || ch == 'H') {
				bigint_bn_AssignInt(object, 0);
				state = TOKEN_HEX;
			}
			else if (ch == 'o' || ch == 'O') {
				bigint_bn_AssignInt(object, 0);
				state = TOKEN_OCT;
			}
			else if (ch == 'd' || ch == 'D') {
				bigint_bn_AssignInt(object, 0);
				state = TOKEN_DEC;
			}
			else if (ch == ' ' || ch == '\t' || ch == '\r' || ch == '\n') {

			}
			else {
				return -1;
			}
		}break;
		case TOKEN_BIN: {
			if (ch == '0' || ch == '1') {
				bigint_bn_MulInt(object, 2);
				bigint_bn_AddInt(object, ch - '0');
				numvalid = 1;
			}
			else if (ch == '_') {
			}
			else {
				goto lastnum;
			}
		}break;
		case TOKEN_OCT: {
			if (ch >= '0' && ch <= '7') {
				bigint_bn_MulInt(object, 8);
				bigint_bn_AddInt(object, ch - '0');
				numvalid = 1;
			}
			else if (ch == '_') {
			}
			else {
				goto lastnum;
			}
		}break;
		case TOKEN_DEC: {
			if (ch >= '0' && ch <= '9') {
				bigint_bn_MulInt(object, 10);
				bigint_bn_AddInt(object, ch - '0');
				numvalid = 1;
			}
			else {
				goto lastnum;
			}
		}break;
		case TOKEN_HEX: {
			if ((ch >= '0' && ch <= '9') || (ch >= 'a' && ch <= 'f') || (ch >= 'A' && ch <= 'F')) {
				numvalid = 1;
				bigint_bn_MulInt(object, 16);
				if (ch > '0' && ch <= '9')
					bigint_bn_AddInt(object, ch - '0');
				else if (ch >= 'a' && ch <= 'f')
					bigint_bn_AddInt(object, ch - 'a' + 10);
				else if (ch >= 'A' && ch <= 'F')
					bigint_bn_AddInt(object, ch - 'A' + 10);
			}
			else if (ch == '_') {
			}
			else {
				goto lastnum;
			}

		}break;
		}
		strt++;
	}
	if (numvalid == 0)
		return -1;
lastnum:
	if (nstr != NULL)
		*nstr = strt;
	bigint_bn_SetWidth(object, pobj->width, 1);
	return 0;
}

static int bigint_bn_AssignInt(HOBJECT object, int value)
{ 
	sBigInteger* pobj;
	int i, width;
	pobj = (sBigInteger*)objectThis(object);
	width = pobj->width;
	bigint_bn_SetWidth(object, 32, 0);
	pobj->buf[0] = *(unsigned int*)&value;
	bigint_bn_SetWidth(object, width, 1);
	return 0;
}

static int bigint_bn_AssignInt64(HOBJECT object, long long value)
{ 
	sBigInteger* pobj;
	int i, width;
	pobj = (sBigInteger*)objectThis(object);
	width = pobj->width;
	bigint_bn_SetWidth(object, 64, 0);
	pobj->buf[0] = (*(unsigned long long *)(&value)) & CELL_MASK;
	pobj->buf[1] = ((*(unsigned long long*)(&value)) >> CELL_WIDTH) & CELL_MASK;
	bigint_bn_SetWidth(object, width, 1);
	return 0;
}

static int bigint_bn_Assign(HOBJECT object, HOBJECT src)
{ 
	sBigInteger* pobj;
	sBigInteger* psrc;
	int i;
	pobj = (sBigInteger*)objectThis(object);
	if (!objectIsClass(src, CLSID_BIGINTEGER)) {
		return -1;
	}
	psrc = (sBigInteger*)objectThis(src);
	if (pobj->buflen < psrc->buflen) {
		unsigned int* buf;
		buf = (unsigned int*)malloc(psrc->buflen * (CELL_WIDTH / 8));
		if (buf == NULL)
			return -2;
		free(pobj->buf);
		pobj->buf = buf;
	}
	pobj->buflen = psrc->buflen;
	pobj->width = psrc->width;
	for (i = 0; i < pobj->buflen; i++)
		pobj->buf[i] = psrc->buf[i];
	return 0;
}

static int bigint_bn_AssignSub(HOBJECT object, HOBJECT src, int from, int width)
{
	bigint_bn_Assign(object, src);
	bigint_bn_SAR(object, from);
	bigint_bn_SetWidth(object, width, 1);
	return 0;
}

static int bigint_bn_Bind(HOBJECT object, HOBJECT src)
{
	int width = bigint_bn_GetWidth(src);
	bigint_bn_SetWidth(object, bigint_bn_GetWidth(object) + width, 0);
	bigint_bn_SHL(object, width);
	bigint_bn_Or(object, src);
	return 0;
}

static int bigint_bn_Abs(HOBJECT object)
{
	if (bigint_bn_IsNeg(object))
		bigint_bn_Neg(object);
	return 0;
}

static int bigint_bn_Neg(HOBJECT object)
{ 
	/* һ */
	bigint_bn_Not(object);
	bigint_bn_AddInt(object, 1);
	return 0;
}

static int bigint_bn_AddInt(HOBJECT object, int value)
{ 
	unsigned long long temp;
	int ind;
	unsigned int v;
	sBigInteger* pobj;
	pobj = (sBigInteger*)objectThis(object);
	temp = pobj->buf[0];
	temp += *(unsigned int*)&value;
	pobj->buf[0] = temp & CELL_MASK;
	temp >>= CELL_WIDTH;
	ind = 1;
	v = 0;
	if (value < 0)
		v = CELL_MASK;
	while (temp != 0) {
		if (ind >= pobj->buflen)
			break;
		temp += v;
		temp += pobj->buf[ind];
		pobj->buf[ind] = temp & CELL_MASK;
		temp >>= CELL_WIDTH;
		ind++;
	}
	return 0;
}

static int bigint_bn_Add(HOBJECT object, HOBJECT src)
{ 
	unsigned long long temp;
	sBigInteger* pobj;
	sBigInteger* psrc;
	int widthobj, widthsrc, wobj;
	int i;
	pobj = (sBigInteger*)objectThis(object);
	if (!objectIsClass(src, CLSID_BIGINTEGER)) {
		return -1;
	}
	psrc = (sBigInteger*)objectThis(src);
	widthobj = pobj->width;
	widthsrc = psrc->width;
	wobj = pobj->buflen * CELL_WIDTH;
	bigint_bn_SetWidth(object, wobj, 1);
	if (widthsrc < wobj)
		bigint_bn_SetWidth(src, wobj, 1);
	temp = 0;
	for (i = 0; i < pobj->buflen; i++) {
		temp += pobj->buf[i];
		temp += psrc->buf[i];
		pobj->buf[i] = temp & CELL_MASK;
		temp >>= CELL_WIDTH;
	}
	bigint_bn_SetWidth(object, widthobj, 0);
	if (widthsrc < wobj)
		bigint_bn_SetWidth(src, widthsrc, 0);
	return 0;
}

static int bigint_bn_SubInt(HOBJECT object, int value)
{ 
	return bigint_bn_AddInt(object, -value);
}

static int bigint_bn_Sub(HOBJECT object, HOBJECT src)
{ 
	sBigInteger** temp;
	temp = bigintegerCreate(64);
	bigint_bn_Assign(temp, src);
	bigint_bn_Neg(temp);
	bigint_bn_Add(object, temp);
	objectRelease(temp);
	return 0;
}

static int bigint_bn_MulInt(HOBJECT object, int value)
{ 
	sBigInteger** temp;
	temp = bigintegerCreate(32);
	bigint_bn_AssignInt(temp, value);
	bigint_bn_Mul(object, temp);
	objectRelease(temp);
	return 0;
}

static int bigint_bn_Mul(HOBJECT object, HOBJECT src)
{ 
	sBigInteger* pobj;
	sBigInteger* psrc;
	unsigned int* buf;
	int i, j;
	int widthobj, widthsrc, wobj;
	pobj = (sBigInteger*)objectThis(object);
	if (!objectIsClass(src, CLSID_BIGINTEGER)) {
		return -1;
	}
	psrc = (sBigInteger*)objectThis(src);
	buf = (unsigned int*)malloc(pobj->buflen * CELL_WIDTH/8);
	if (buf == NULL)
		return -2;

	widthobj = pobj->width;
	widthsrc = psrc->width;
	wobj = pobj->buflen * CELL_WIDTH;
	bigint_bn_SetWidth(object, wobj, 1);
	if (widthsrc < wobj)
		bigint_bn_SetWidth(src, wobj, 1);

	for (i = 0; i < pobj->buflen; i++)
		buf[i] = 0;
	for (i = 0; i < pobj->buflen; i++) {
		unsigned long long addin;
		addin = 0;
		for (j = 0; j < psrc->buflen; j++) {
			unsigned long long m0, m1;
			if (i + j >= pobj->buflen)
				break;
			m0 = pobj->buf[i];
			m1 = psrc->buf[j];
			m0 = m0 * m1 + addin + buf[i+j];
			buf[i + j] = m0 & CELL_MASK;
			addin = m0 >> CELL_WIDTH;
		}
	}
	free(pobj->buf);
	pobj->buf = buf;
	bigint_bn_SetWidth(object, widthobj, 0);
	if (widthsrc < wobj)
		bigint_bn_SetWidth(src, widthsrc, 0);
	return 0;
}

static int bigint_bn_DivInt(HOBJECT object, int value)
{ 
	sBigInteger* pobj;
	pobj = (sBigInteger*)objectThis(object);
	return 0;
}

static int bigint_bn_Div(HOBJECT object, HOBJECT src)
{ 
	sBigInteger* pobj;
	pobj = (sBigInteger*)objectThis(object);
	return 0;
}

static int bigint_bn_SHL(HOBJECT object, int bits)
{ 
	sBigInteger* pobj;
	int ifrom, ito, i;
	unsigned long long current, next;
	pobj = (sBigInteger*)objectThis(object);
	if (bits == 0)
		return 0;
	if (bits < 0)
		return bigint_bn_SHR(object, -bits);
	ito = bits / CELL_WIDTH;
	bits -= ito * CELL_WIDTH;
	ifrom = 0;
	for (i = pobj->buflen-1; i>=0; i--) {
		current = 0;
		if (i >= ito) {
			current = pobj->buf[i - ito];
			current <<= CELL_WIDTH;
		}
		if (i >= ito+1) {
			next = pobj->buf[i - ito -1];
			current |= next;
		}
		current >>= CELL_WIDTH - bits;
		pobj->buf[i] = (unsigned int)(current & CELL_MASK);
	}
	return 0;
}

static int bigint_bn_SHR(HOBJECT object, int bits)
{
	sBigInteger* pobj;
	int bc;
	int ifrom, ito, i, zerolen;
	unsigned long long current, next;
	pobj = (sBigInteger*)objectThis(object);
	if (bits == 0)
		return 0;
	if (bits < 0)
		return bigint_bn_SHL(object, -bits);
	ito = 0;
	ifrom = bits / CELL_WIDTH;
	if (ifrom >= pobj->buflen) {
		memset(pobj->buf, 0, pobj->buflen * (CELL_WIDTH/8));
		return 0;
	}
	bits -= ifrom * CELL_WIDTH;
	if (ifrom+1 < pobj->buflen) {
		current = pobj->buf[ifrom+1];
		current <<= CELL_WIDTH;
	}
	else {
		current = 0;
	}
	current |= pobj->buf[ifrom];
	current >>= bits;
	for (i = ifrom + 1; i < pobj->buflen-1; i++) {
		pobj->buf[ito++] = (unsigned int)(current & CELL_MASK);
		current >>= CELL_WIDTH;
		next = pobj->buf[i + 1];
		next <<= CELL_WIDTH-bits;
		current |= next;
	}
	pobj->buf[ito++] = (unsigned int)(current & CELL_MASK);
	while (ito < pobj->buflen)
		pobj->buf[ito++] = 0;
	return 0;
}

static int bigint_bn_SAL(HOBJECT object, int bits)
{
	return bigint_bn_SHL(object, bits);
}

static int bigint_bn_SAR(HOBJECT object, int bits)
{
	sBigInteger* pobj;
	int bc, sign;
	int ifrom, ito, i, zerolen;
	unsigned long long current, next;
	pobj = (sBigInteger*)objectThis(object);
	if (bits == 0)
		return 0;
	if (bits < 0)
		return bigint_bn_SHL(object, -bits);
	bc = pobj->width / CELL_WIDTH;
	sign = pobj->buf[bc] & (1 << ((pobj->width-1) & (CELL_WIDTH - 1)));
	ito = 0;
	ifrom = bits / CELL_WIDTH;
	if (ifrom >= pobj->buflen) {
		for (i = 0;i<pobj->buflen;i++)
			pobj->buf[i] = sign ? CELL_MASK : 0;
		return 0;
	}
	bits -= ifrom * CELL_WIDTH;
	if (ifrom + 1 < pobj->buflen) {
		current = pobj->buf[ifrom + 1];
		current <<= CELL_WIDTH;
	}
	else {
		current = 0;
	}
	current |= pobj->buf[ifrom];
	current >>= bits;
	for (i = ifrom + 1; i < pobj->buflen - 1; i++) {
		pobj->buf[ito++] = (unsigned int)(current & CELL_MASK);
		current >>= CELL_WIDTH;
		next = pobj->buf[i + 1];
		next <<= CELL_WIDTH - bits;
		current |= next;
	}
	next = sign ? CELL_MASK : 0;
	next <<= CELL_WIDTH - bits;
	current |= next;
	pobj->buf[ito++] = (unsigned int)(current & CELL_MASK);
	while (ito < pobj->buflen)
		pobj->buf[ito++] = sign ? CELL_MASK : 0;
	return 0;
}


static int bigint_bn_Not(HOBJECT object)
{
	int i;
	sBigInteger* pobj;
	pobj = (sBigInteger*)objectThis(object);
	for (i = 0; i < pobj->buflen; i++)
		pobj->buf[i] = ~pobj->buf[i];
	return 0;
}

static int bigint_bn_uAnd(HOBJECT object)
{
	sBigInteger* pobj;
	pobj = (sBigInteger*)objectThis(object);
	return 0;
}

static int bigint_bn_uOr(HOBJECT object)
{
	sBigInteger* pobj;
	pobj = (sBigInteger*)objectThis(object);
	return 0;
}

static int bigint_bn_uXor(HOBJECT object)
{
	sBigInteger* pobj;
	pobj = (sBigInteger*)objectThis(object);
	return 0;
}

static int bigint_bn_And(HOBJECT object, HOBJECT src)
{ 
	sBigInteger* pobj;
	sBigInteger* psrc;
	int i;
	pobj = (sBigInteger*)objectThis(object);
	if (!objectIsClass(src, CLSID_BIGINTEGER)) {
		return -1;
	}
	psrc = (sBigInteger*)objectThis(src);
	for (i = 0; i < pobj->buflen && i < psrc->buflen; i++) {
		pobj->buf[i] &= psrc->buf[i];
	}
	for (; i < pobj->buflen; i++) {
		pobj->buf[i] = 0;
	}
	return 0;
}

static int bigint_bn_Or(HOBJECT object, HOBJECT src)
{ 
	sBigInteger* pobj;
	sBigInteger* psrc;
	int i;
	pobj = (sBigInteger*)objectThis(object);
	if (!objectIsClass(src, CLSID_BIGINTEGER)) {
		return -1;
	}
	psrc = (sBigInteger*)objectThis(src);
	for (i = 0; i < pobj->buflen && i < psrc->buflen; i++) {
		pobj->buf[i] |= psrc->buf[i];
	}
	return 0;
}

static int bigint_bn_Xor(HOBJECT object, HOBJECT src)
{
	sBigInteger* pobj;
	sBigInteger* psrc;
	int i;
	pobj = (sBigInteger*)objectThis(object);
	if (!objectIsClass(src, CLSID_BIGINTEGER)) {
		return -1;
	}
	psrc = (sBigInteger*)objectThis(src);
	for (i = 0; i < pobj->buflen && i < psrc->buflen; i++) {
		pobj->buf[i] ^= psrc->buf[i];
	}
	for (; i < pobj->buflen; i++) {
		pobj->buf[i] ^= 0;
	}
	return 0;
}

static int bigint_bn_IsZero(HOBJECT object)
{
	sBigInteger* pobj;
	int i;
	pobj = (sBigInteger*)objectThis(object);
	for (i = 0; i < pobj->buflen; i++)
		if (pobj->buf[i] != 0)
			return 0;
	return 1;
}

static int bigint_bn_IsNeg(HOBJECT object)
{ 
	sBigInteger* pobj;
	int bc, sign;
	pobj = (sBigInteger*)objectThis(object);
	bc = pobj->width / CELL_WIDTH;
	sign = pobj->buf[bc] & (1 << ((pobj->width-1) & (CELL_WIDTH - 1)));
	return sign?1:0;
}

static int bigint_bn_IsEQ(HOBJECT object, HOBJECT src)
{ 
	sBigInteger* pobj;
	sBigInteger* psrc;
	int i;
	int bc, sign, signsrc;
	pobj = (sBigInteger*)objectThis(object);
	if (!objectIsClass(src, CLSID_BIGINTEGER)) {
		return 0;
	}
	psrc = (sBigInteger*)objectThis(src);
	bc = pobj->width / CELL_WIDTH;
	sign = pobj->buf[bc] & (1 << ((pobj->width-1) & (CELL_WIDTH - 1)));
	bc = psrc->width / CELL_WIDTH;
	signsrc = psrc->buf[bc] & (1 << ((psrc->width-1) & (CELL_WIDTH - 1)));
	if (sign != signsrc) {
		return 0;
	}
	for (i = 0; i < pobj->buflen && i < psrc->buflen; i++) {
		if (pobj->buf[i] != psrc->buf[i])
			return 0;
	}
	for (; i < pobj->buflen; i++) {
		if (sign == 0 && pobj->buf[i] != 0)
			return 0;
		else if (sign != 0 && pobj->buf[i] != CELL_MASK)
			return 0;
	}
	for (; i < psrc->buflen; i++) {
		if (sign == 0 && psrc->buf[i] != 0)
			return 0;
		else if (sign != 0 && psrc->buf[i] != CELL_MASK)
			return 0;
	}
	return 1;
}

static int bigint_bn_IsLE(HOBJECT object, HOBJECT src)
{ 
	sBigInteger* pobj;
	sBigInteger* psrc;
	int i;
	int bc, sign, signsrc;

	pobj = (sBigInteger*)objectThis(object);
	if (!objectIsClass(src, CLSID_BIGINTEGER)) {
		return 0;
	}
	psrc = (sBigInteger*)objectThis(src);
	bc = pobj->width / CELL_WIDTH;
	sign = pobj->buf[bc] & (1 << ((pobj->width-1) & (CELL_WIDTH - 1)));
	bc = psrc->width / CELL_WIDTH;
	signsrc = psrc->buf[bc] & (1 << ((psrc->width-1) & (CELL_WIDTH - 1)));
	if (sign == 0 && signsrc != 0) {
		return 0;
	}
	else if (sign != 0 && signsrc == 0) {
		return 1;
	}
	
	bc = pobj->buflen;
	if (bc < psrc->buflen)
		bc = psrc->buflen;
	for (i = bc - 1; i >= pobj->buflen; i--) {
		if (sign != 0) {
			if (psrc->buf[i] != 0xffffffff)
				return 0;
		}
		else {
			if (psrc->buf[0] != 0)
				return 1;
		}
	}
	for (; i >= psrc->buflen; i--) {
		if (sign != 0) {
			if (pobj->buf[i] != 0xffffffff)
				return 1;
		}
		else {
			if (pobj->buf[0] != 0)
				return 0;
		}
	}
	for (; i >= 0; i--) {
		if (sign != 0) {
			if (pobj->buf[i] > psrc->buf[i])
				return 1;
			else if (pobj->buf[i] < psrc->buf[i])
				return 0;
		}
		else {
			if (pobj->buf[i] < psrc->buf[i])
				return 1;
			if (pobj->buf[i] > psrc->buf[i])
				return 0;
		}
	}
	return 1;
}

static int bigint_bn_IsLT(HOBJECT object, HOBJECT src)
{ 
	sBigInteger* pobj;
	sBigInteger* psrc;
	int i;
	int bc, sign, signsrc;

	pobj = (sBigInteger*)objectThis(object);
	if (!objectIsClass(src, CLSID_BIGINTEGER)) {
		return 0;
	}
	psrc = (sBigInteger*)objectThis(src);
	bc = pobj->width / CELL_WIDTH;
	sign = pobj->buf[bc] & (1 << ((pobj->width-1) & (CELL_WIDTH - 1)));
	bc = psrc->width / CELL_WIDTH;
	signsrc = psrc->buf[bc] & (1 << ((psrc->width-1) & (CELL_WIDTH - 1)));
	if (sign == 0 && signsrc != 0) {
		return 0;
	}
	else if (sign != 0 && signsrc == 0) {
		return 1;
	}

	bc = pobj->buflen;
	if (bc < psrc->buflen)
		bc = psrc->buflen;
	for (i = bc - 1; i >= pobj->buflen; i--) {
		if (sign != 0) {
			if (psrc->buf[i] != 0xffffffff)
				return 0;
		}
		else {
			if (psrc->buf[0] != 0)
				return 1;
		}
	}
	for (; i >= psrc->buflen; i--) {
		if (sign != 0) {
			if (pobj->buf[i] != 0xffffffff)
				return 1;
		}
		else {
			if (pobj->buf[0] != 0)
				return 0;
		}
	}
	for (; i >= 0; i--) {
		if (sign != 0) {
			if (pobj->buf[i] > psrc->buf[i])
				return 1;
			else if (pobj->buf[i] < psrc->buf[i])
				return 0;
		}
		else {
			if (pobj->buf[i] < psrc->buf[i])
				return 1;
			if (pobj->buf[i] > psrc->buf[i])
				return 0;
		}
	}
	return 0;
}

IBigNumber** bigintegerCreate(int width)
{
	int ret;
	IBigNumber** bn;
	PARAMITEM param;
	A_u_t_o_registor_bigint();
	param.name = PARAMID_BIGINTEGERWIDTH;
	param.i32value = width;
	ret = objectCreateEx(CLSID_BIGINTEGER, &param, 1, IID_BIGNUMBER, (const void**)&bn);
	return bn;
}
